# Copyright (c) 2020 Xiaozhe Yao & AICAMP.CO.,LTD
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

import os
from mlpm.server import run_server
from mlpm.logger import StepLogger

class Solver(object):
    def __init__(self, pretrained_toml=None):
        self._isReady = False
        self.bundle = None
        self._hyperparameters = {}
        self._enable_train = False
        if hasattr(self.__class__, 'train') and callable(getattr(self.__class__, 'train')):
            self._enable_train = True
            self.logger = StepLogger(os.environ('LOG_FILE'))

    @property
    def enable_train(self):
        return self._enable_train

    @property
    def is_ready(self):
        return self._isReady

    @property
    def hyperparamters(self):
        return self._hyperparameters

    def ready(self):
        self._isReady = True

    def start(self, port=None):
        print('Server will run on port: ' + str(port))
        run_server(self, port)

    def infer(self, data):
        raise NotImplementedError

    def train(self, data):
        raise NotImplementedError



# Copyright (c) 2020 Xiaozhe Yao & AICAMP.CO.,LTD
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT
import json
import os

from mlpm.config import GetConfig


class Logger(object):
    def __init__(self, config, target):
        self.target = target
        self.remote_log = config['remote_log']

    def write(self, logObject):
        with open(self.target, 'w+') as f:
            json.dump(logObject, f)

    def append(self, logObject):
        if os.stat(self.target).st_size == 0:
            self.write(logObject)
        else:
            with open(self.target, 'w+') as f:
                data = json.load(f)
                data.append(logObject)
                json.dump(data, f)

class StepLogger(object):
    def __init__(self, logFile):
        self.logFile = logFile
    
    def append(self, logObject):
        if os.stat(self.target).st_size == 0:
            self.write(logObject)
        else:
            with open(self.target, 'w+') as f:
                data = json.load(f)
                data.append(logObject)
                json.dump(data, f)



# Copyright (c) 2020 Xiaozhe Yao & AICAMP.CO.,LTD
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

#coding:utf-8
import os
from pathlib import Path


def GetHomeDir():
    return os.path.join(str(Path.home()), '.autoai', '.aid')


def GetLogDir():
    return os.path.join(GetHomDir(), 'logs')


def GetConfig():
    with open(os.path.join(GetHomeDir(), 'config.toml'), 'r') as f:
        config = toml.load(f)
        return config



# Copyright (c) 2020 Xiaozhe Yao & AICAMP.CO.,LTD
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT
# coding:utf-8
import os
import traceback
from sanic import Sanic, request, response
from sanic.response import json
from werkzeug.datastructures import ImmutableMultiDict
from werkzeug.utils import secure_filename

from mlpm.app import aidserver
from mlpm.utility import str2bool

def handle_post_solver_train_or_infer(request, upload_folder, request_type):
    config = ImmutableMultiDict(request.form)
    data = config.to_dict()
    results = {}
    if 'file' in request.files:
        filename = secure_filename(request.files["file"][0].name)
        # make sure the UPLOAD_FOLDER exsits
        if not os.path.isdir(upload_folder):
            os.makedirs(upload_folder)
        file_abs_path = os.path.join(upload_folder, filename)
        with open(file_abs_path,"wb") as file:
            file.write(request.files["file"][0].body)
        file.close()
        data['input_file_path'] = file_abs_path
    try:
        if request_type == "infer":
            results = aidserver.solver.infer(data)
        elif request_type == "train":
            results = aidserver.solver.train(data)
        else:
            raise NotImplementedError
        if 'delete_after_process' in data:
            if str2bool(data['delete_after_process']):
                os.remove(file_abs_path)
        print(results)
        return response.json(results, status=200)
    except Exception as e:
        traceback.print_exc()
        return response.json({"error": str(e), "code": "500"}, status=500)



# Copyright (c) 2020 Xiaozhe Yao & AICAMP.CO.,LTD
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

#coding:utf-8

from sanic import request, response
from sanic.response import json
from werkzeug.datastructures import ImmutableMultiDict
from werkzeug.utils import secure_filename

from mlpm.handler import handle_post_solver_train_or_infer
from mlpm.utility import get_available_port, str2bool

from mlpm.app import aidserver

UPLOAD_INFER_FOLDER = './temp/infer'
UPLOAD_TRAIN_FOLDER = './temp/train'


@aidserver.route("/", methods=["GET"])
async def ping(request):
    return response.text('Hello world!', status=200)


@aidserver.route("/infer", methods=["GET", "POST"])
async def infer(request):
    if request.method == 'POST':
        return handle_post_solver_train_or_infer(request, UPLOAD_INFER_FOLDER,
                                          "infer")


@aidserver.route("/train", methods=["GET", "POST"])
async def train(request):
    if request.method == "POST":
        return handle_post_solver_train_or_infer(request, UPLOAD_TRAIN_FOLDER,
                                          "train")


def run_server(solver, port=None):
    if port is None:
        port = get_available_port()
    aidserver.solver = solver
    aidserver.run(host='0.0.0.0', port=port, workers=4)



# Copyright (c) 2020 Xiaozhe Yao & AICAMP.CO.,LTD
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

#coding:utf-8


class ThreadPrinter:
    def __init__(self, filename):
        self.fhs = {}
        self.filename = filename

    def write(self, value):
        try:
            f = self.fhs.get(threading.current_thread().uuid)
            if f is None:
                f = open(self.filename, 'a')
            f.write(value)
            self.fhs[threading.current_thread().uuid] = f
            f.flush()
        except IOError as identifier:
            logging.warn(identifier)


class TracedThread(threading.Thread):
    def __init__(self, *args, **keywords):
        threading.Thread.__init__(self, *args, **keywords)
        self.id = None
        self.killed = False
        self.oldout = None

    @property
    def uuid(self):
        return self.id

    def start(self):
        if self.id is None:
            self.id = str(uuid.uuid4)
        full_log_path = os.path.join(getLogDir(), self.id + ".full.log")
        self.__run_backup = self.run
        self.run = self.__run
        sys.stdout = ThreadPrinter(full_log_path)
        threading.Thread.start(self)

    def __run(self):
        sys.settrace(self.globaltrace)
        self.__run_backup()
        self.run = self.__run_backup

    def globaltrace(self, frame, event, arg):
        if event == 'call':
            return self.localtrace
        else:
            return None

    def localtrace(self, frame, event, arg):
        if self.killed:
            if event == 'line':
                raise SystemExit()
        return self.localtrace

    def kill(self):
        print('Terminating ' + self.id + '...')
        sys.stdout = self.oldout
        self.killed = True



# Copyright (c) 2020 Xiaozhe Yao & AICAMP.CO.,LTD
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT


class Metrics(object):
    def __init__(self):
        self.data = []

    def add(self, value):
        # value is assumed to be a json object, rather than a plain text
        # e.g. {step: 0, acc:1}
        self.data.append(value)

    @property
    def output(self):
        return {}



import setuptools

long_description = "For full description, please go to [Documentation](https://aid.autoai.org)"

setuptools.setup(
    name="mlpm",
    version="1.0.0.c",
    author="Xiaozhe Yao",
    author_email="xiaozhe.yaoi@gmail.com",
    description="Machine Learning Package Manager",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://aid.autoai.org",
    packages=['mlpm'],
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Image Recognition",
        "Topic :: Software Development",
    ],
    install_requires=[
        "sanic",
        "toml",
        "requests",
        "pillow",
        "numpy",
    ],
    project_urls={
        "Bug Tracker": "https://github.com/autoai-org/aid/issues",
        "Documentation": "https://aid.autoai.org",
        "Source Code": "https://github.com/autoai-org/aid",
    },
)


# Copyright (c) 2020 Xiaozhe Yao & AICAMP.CO.,LTD
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

import socket


def str2bool(v):
    return str(v).lower() in ("true", "false", "yes", "t", "1")


def _isPortOpen(port):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    result = s.connect_ex(('127.0.0.1', port))
    s.close()
    if result == 0:
        return True
    else:
        return False


def get_available_port(start=8080):
    port = start
    while True:
        if _isPortOpen(port):
            port = port + 1
        else:
            break
    return port



# Copyright (c) 2020 Xiaozhe Yao & AICAMP.CO.,LTD
# 
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT
#coding:utf-8

from sanic import Sanic

aidserver = Sanic()
